//
//  FWVideoFeed.swift
//  react-native-firework-sdk
//
//  Created by Jeff Zheng on 2021/12/22.
//
//  FWVideoFeed wrap the VideoFeedViewController class (which need to embed subview throught 'addChildViewController') which can simply embed host view throught 'addSubview'

import UIKit
import FireworkVideo

@objc
public enum VideoFeedMode: Int {
    case row, column, grid
}

@objc
public enum VideFeedSourceType: Int {
    case discover, channel, playlist, playlistGroup, dynamicContent
}

public protocol VideoFeedViewDelegate: AnyObject {
    /// Called once after a video feed has been successfully loaded
    func videoFeedDidLoadFeed(_ view: VideoFeed)
    /// Called if the video feed failed to load
    func videoFeed(_ view: VideoFeed,
                   didFailToLoadFeed error: VideoFeedError)
}

@objc
public class VideoFeed: UIView, VideoFeedViewControllerDelegate {
    @objc public var sourceType: VideFeedSourceType = .discover
    @objc public var channel: String = ""
    @objc public var playlist: String = ""
    @objc public var playlistGroup: String = ""
    @objc public var dynamicContentParameters: [String: [String]] = [:]

    @objc public var mode: VideoFeedMode = .row
    @objc public var feedViewConfig: VideoFeedConfiguration? {
        didSet {
            guard let feedVC = feedVC else {
                return
            }
            feedVC.viewConfiguration = convertToVideoFeedContentConfiguration()
            feedVC.layout = videoFeedLayout
        }
    }
    @objc public var playerViewConfig: VideoPlayerConfiguration? {
       didSet {
           guard let feedVC = feedVC else {
               return
           }
           feedVC.viewConfiguration = convertToVideoFeedContentConfiguration()
       }
    }
//    @objc var onVideoFeedLoadFinished: RCTBubblingEventBlock?
//    @objc var onVideoFeedClick: RCTBubblingEventBlock?
    
    public weak var delegate: VideoFeedViewDelegate?
    
    private var feedVC: VideoFeedViewController?
    private var source: VideoFeedContentSource {
        switch sourceType {
        case .discover:
            return .discover
        case .channel:
            return .channel(channelID: channel)
        case .playlist:
            return .channelPlaylist(channelID: channel, playlistID: playlist)
        case .playlistGroup:
            return .playlistGroup(groupID: playlistGroup)
        case .dynamicContent:
            return .dynamicContent(channelID: channel, parameters: dynamicContentParameters)
        }
    }

    private var videoFeedLayout: VideoFeedLayout {
        var resultLayout: VideoFeedLayout?
        switch mode {
        case .row:
            let layout = VideoFeedHorizontalLayout()
            resultLayout = layout
            break
        case .column:
            let layout = VideoFeedGridLayout()
            layout.numberOfColumns = 1
            resultLayout = layout
            break
        case .grid:
            let layout = VideoFeedGridLayout()
            if let gridColumns = feedViewConfig?.gridColumns,
               gridColumns > 0 {
                layout.numberOfColumns = gridColumns
            } else {
                layout.numberOfColumns = 2
            }
            resultLayout = layout
            break
        }
        if let config = feedViewConfig {
            if let aspectRatio = config.aspectRatio {
                if let horizontalLayout = resultLayout as? VideoFeedHorizontalLayout {
                    horizontalLayout.itemWidthRatio = aspectRatio
                } else if let gridLayout = resultLayout as? VideoFeedGridLayout {
                    gridLayout.itemWidthRatio = aspectRatio
                }
            }
            if let contentPadding = config.contentPadding {
                if let top = contentPadding.top {
                    resultLayout?.contentInsets.top = top
                }
                if let right = contentPadding.right {
                    resultLayout?.contentInsets.right = right
                }
                if let bottom = contentPadding.bottom {
                    resultLayout?.contentInsets.bottom = bottom
                }
                if let left = contentPadding.left {
                    resultLayout?.contentInsets.left = left
                }
            }

            if let itemSpacing = config.itemSpacing {
                resultLayout?.itemSpacing = itemSpacing
            }
        }

        return resultLayout ?? VideoFeedHorizontalLayout()
    }
    
    public override func layoutSubviews() {
        super.layoutSubviews()
        
        embed()
    }
    
    private func embed() {
        guard let parentVC = parentViewController else {
            return
        }
        guard self.feedVC == nil else {
            return
        }
        
        let feedVC = VideoFeedViewController(
            layout: videoFeedLayout,
            source: source
        )
        var viewConfiguration = convertToVideoFeedContentConfiguration()
        if viewConfiguration.itemView.autoplay.isEnabled {
            viewConfiguration.itemView.autoplay.isEnabled = false
            feedVC.viewConfiguration = viewConfiguration
            DispatchQueue.main.async {
                if feedVC.viewConfiguration == viewConfiguration {
                    viewConfiguration.itemView.autoplay.isEnabled = true
                    feedVC.viewConfiguration = viewConfiguration
                }
            }
        } else {
            feedVC.viewConfiguration = viewConfiguration
        }
        feedVC.delegate = self
        self.feedVC = feedVC

        parentVC.attachChild(feedVC, to: self)
    }
    
    public override func removeFromSuperview() {
        super.removeFromSuperview()

        guard let feedVC = self.feedVC else {
            return
        }
        
        feedVC.detachFromParent()
        self.feedVC = nil
    }
    
    @objc
    public func refresh() {
        guard let feedVC = self.feedVC else {
            return
        }
        feedVC.refresh()
    }
    
    //MARK: - VideoFeedViewControllerDelegate
    public func videoFeedDidLoadFeed(_ viewController: VideoFeedViewController) {
        guard let delegate = self.delegate else {
            return
        }
        delegate.videoFeedDidLoadFeed(self)
    }
    
    public func videoFeed(_ viewController: VideoFeedViewController, didFailToLoadFeed error: VideoFeedError) {
        guard let delegate = self.delegate else {
            return
        }
        delegate.videoFeed(self, didFailToLoadFeed: error)
    }
}

extension VideoFeed {
    private func convertToVideoFeedContentConfiguration() -> VideoFeedContentConfiguration {
        var videoConfig = VideoFeedContentConfiguration()
        //set default value so that behavior will the same with Android
        videoConfig.itemView.title.isHidden = false
        videoConfig.itemView.titleLayoutConfiguration.titlePosition = .nested
        
        let vfcConfig = VideoFeed.convertToVideoFeedItemContentConfiguration(feedViewConfig)
        let vpcConfig = VideoFeed.convertToVideoPlayerContentConfiguration(playerViewConfig)
        if let vfcConfig = vfcConfig {
            videoConfig.itemView = vfcConfig
        }
        if let vpcConfig = vpcConfig {
            videoConfig.playerView = vpcConfig
        }
        
        if let backgroundColor = feedViewConfig?.backgroundColor {
            videoConfig.backgroundColor = backgroundColor.uicolor()
        }
        if let shareBaseUrl = FireworkSDKModulePlugin.ShareBaseURLConfiguration.shareBaseURL {
            videoConfig.playerView.shareButton.behavior.baseURL = URL(string:shareBaseUrl)
        }

        if let adBadge = FireworkSDKModulePlugin.FWAdBadgeConfiguration.fwAdBadgeConfiguration  {
            videoConfig.adBadge = adBadge
        }
        
        //set a value different from default value to compatible with FireworkVideo SDK misalign bug
        videoConfig.itemView.titleLayoutConfiguration.insets = UIEdgeInsets(top: 7, left: 7, bottom: 7, right: 7)
        
        return videoConfig
    }

    private static func convertToVideoFeedItemContentConfiguration(_ config: VideoFeedConfiguration?) -> VideoFeedItemContentConfiguration? {
        guard let config = config else {
            return nil
        }
        
        var vfcConfig = VideoFeedItemContentConfiguration()
        //set default value so that behavior will the same with Android
        vfcConfig.title.isHidden = false
        vfcConfig.titleLayoutConfiguration.titlePosition = .nested
        
        if let cornerRadius = config.cornerRadius {
            vfcConfig.cornerRadius = cornerRadius
        }
        if let title = config.title {
            if let hidden = title.hidden {
                vfcConfig.title.isHidden = hidden
            }
            if let textcolor = title.textColor {
                vfcConfig.title.textColor = textcolor.uicolor()
            }
            if let fontSize = title.fontSize {
                vfcConfig.title.font = UIFont.systemFont(ofSize: fontSize)
            }
        }
        if let playIcon = config.playIcon {
            if let hidden = playIcon.hidden {
                vfcConfig.playIcon.isHidden = hidden
            }
            if let iconWidth = playIcon.iconWidth {
                vfcConfig.playIcon.iconWidth = iconWidth
            }
        }
        if let position = config.titlePosition {
            switch position {
            case .stacked:
                vfcConfig.titleLayoutConfiguration.titlePosition = .stacked
            default:
                vfcConfig.titleLayoutConfiguration.titlePosition = .nested
            }
        }
        if let showSponsored = config.showSponsored {
            vfcConfig.sponsored.isHidden = !showSponsored
        }
        if let showAdBadge = config.showAdBadge {
            vfcConfig.sponsored.isHidden = !showAdBadge
        }

        if let enableAutoplay = config.enableAutoplay {
            vfcConfig.autoplay.isEnabled = enableAutoplay
        }
        
        return vfcConfig
    }
    
    private static func convertToVideoPlayerContentConfiguration(_ config: VideoPlayerConfiguration?) -> VideoPlayerContentConfiguration? {
        guard let config = config else {
            return nil
        }
        
        var vpcConfig = VideoPlayerContentConfiguration()
        if let playerStyle = config.playerStyle {
            switch playerStyle {
            case .full:
                vpcConfig.playerStyle = .fullBleed
            default:
                vpcConfig.playerStyle = .fit
            }
        }
        if let videoCompleteAction = config.videoCompleteAction {
            switch videoCompleteAction {
            case .loop:
                vpcConfig.videoCompleteAction = .loop
            default:
                vpcConfig.videoCompleteAction = .advanceToNext
            }
        }
        if let showShareButton = config.showShareButton {
            vpcConfig.shareButton.isHidden = !showShareButton
        }
        if let ctaButtonStyle = config.ctaButtonStyle {
            if let backgroundColor = ctaButtonStyle.backgroundColor {
                vpcConfig.ctaButton.contentConfiguration.backgroundColor = backgroundColor.uicolor()
            }
            if let textcolor = ctaButtonStyle.textColor {
                vpcConfig.ctaButton.contentConfiguration.textColor = textcolor.uicolor()
            }
            if let fontSize = ctaButtonStyle.fontSize {
                vpcConfig.ctaButton.contentConfiguration.font = UIFont.systemFont(ofSize: fontSize)
            }
        }
        if let showPlaybackButton = config.showPlaybackButton {
            vpcConfig.playbackButton.isHidden = !showPlaybackButton
        }
        if let showMuteButton = config.showMuteButton {
            vpcConfig.muteButton.isHidden = !showMuteButton
        }

        if let launchBehavior = config.launchBehavior {
            vpcConfig.onFirstLaunch = launchBehavior.behavior()
        }

        if let showBranding = config.showBranding {
            vpcConfig.videoDetail.fireworkAttribution.isHidden = !showBranding
        }
        
        return vpcConfig
    }
}

extension VideoPlayerConfiguration.VideoLaunchBehavior {
    func behavior() -> VideoPlayerContentConfiguration.VideoLaunchBehavior {
        switch self {
        case .muteOnFirstLaunch:
            return .muteOnFirstLaunch
        default:
            return .`default`
        }
    }
}
